@@ -3,8 +3,9 @@ import re
 from dataclasses import dataclass
 from logging import NullHandler
 from pathlib import Path
-from typing import Optional, Any
+from typing import Optional, Any, Union
 from urllib import request
+from urllib.request import HTTPCookieProcessor, Request
 
 import bs4
 from bs4 import BeautifulSoup
@@ -21,14 +22,14 @@ class IMDBAsset:
     genres: set[str]
     persons: dict[str, list[int]]
     awards: dict[str, Any]
-    ratings: dict
+    ratings: dict[str, Any]
     budget: Optional[int]
     synopsis: str
 
 
 class IMDBAssetScraper:
     logger: logging.Logger
-    URL_BASE: str = 'http://www.imdb.com/title/tt'
+    URL_BASE: str = 'https://imdb.com/title/tt'
     dir_cache: Path
 
     def __init__(self, dir_cache: Path):
@@ -36,6 +37,7 @@ class IMDBAssetScraper:
         self.logger = logging.getLogger(__name__)
         self.logger.addHandler(NullHandler())
         self.dir_cache = dir_cache
+        self.header: dict[str, str] = {'User-Agent': 'Mozilla/5.0'}
 
     def process_imdb_movie_id(self, imdb_movie_id: int, use_cache: bool = False) -> IMDBAsset:
         """Get website for a given imdb_movie_id and return parsed asset
@@ -55,28 +57,29 @@ class IMDBAssetScraper:
         """Provide the website for a given imdb_movie_id as a string to be parsed
 
         :param imdb_movie_id: unique ID used by imdb
-        :param use_cache: if true, a already stored string will be used and no request to the website will be made
-        :return: raw string of the website
+        :param use_cache: if true, an already stored string will be used and no request to the website will be made
+        :return: raw string of the website, with all subsites appended
         """
         website_string: bytes = b""
         file_path: Path = Path(self.dir_cache, f"{imdb_movie_id}.imdb_movie")
         if use_cache:
             try:
-                with file_path.open('rb') as file_handle:
-                    website_string = file_handle.read()
-                    self.logger.info(f"Loading cached website for {imdb_movie_id=}.")
+                website_string = file_path.read_bytes()
+                self.logger.info(f"Loading cached website for {imdb_movie_id=}.")
             except FileNotFoundError:
                 self.logger.info(f"{imdb_movie_id=} not found in cache.")
         if not website_string or not use_cache:
             self.logger.info(f"Retrieving website for {imdb_movie_id=}.")
-            url_movie: str = self.URL_BASE + str(imdb_movie_id).zfill(7)
-            website_string = request.urlopen(url_movie).read()
+            url_movie: str = self.URL_BASE + str(imdb_movie_id).zfill(7) + "/"
+            self.logger.debug(f"URL {url_movie=}.")
+            opener = request.build_opener(HTTPCookieProcessor())
+            website_string = opener.open(Request(url_movie, headers=self.header)).read()
             for sub_site in ('parentalguide', 'fullcredits', 'awards', 'business', 'companycredits', 'technical',
                              'keywords', 'plotsummary'):
-                website_sub = request.urlopen(f'{url_movie}/{sub_site}')
+                self.logger.debug(f"Start loading {sub_site=}.")
+                website_sub = opener.open(Request(f'{url_movie}{sub_site}', headers=self.header))
                 website_string += website_sub.read()
-            with file_path.open('wb') as f:
-                f.write(website_string)
+            file_path.write_bytes(website_string)
         return website_string.decode("utf-8")
 
     def parse_webcontent_4_imdb_movie(self, imdb_movie_id: int, website: str) -> IMDBAsset:
@@ -103,22 +106,35 @@ class IMDBAssetScraper:
         return asset_obj
 
     @staticmethod
-    def _parse_rating_from_soup(soup: BeautifulSoup):
+    def _parse_rating_from_soup(soup: BeautifulSoup) -> dict[str, Union[int, float, None]]:
         try:
-            rating_imdb_raw = soup.select('span[class^="AggregateRatingButton__RatingScore"]')[0].get_text()
+            res = soup.select('span[class^="AggregateRatingButton__RatingScore"]')[0]
+            if res:
+                rating_imdb_raw = res.get_text()
+            else:
+                rating_imdb_raw = 0
             rating_imdb_count_raw = soup.select('div[class^="AggregateRatingButton__TotalRatingAmount"]')[0].get_text()
         except IndexError:
-            rating_imdb_raw = soup.find('span', attrs={'itemprop': 'ratingValue'}).get_text()
-            rating_imdb_count_raw = soup.find('span', attrs={'itemprop': 'ratingCount'}).get_text()
-        rating_imdb = float(rating_imdb_raw)
+            try:
+                rating_imdb_raw = soup.find('span', attrs={'itemprop': 'ratingValue'}).get_text()
+                rating_imdb_count_raw = soup.find('span', attrs={'itemprop': 'ratingCount'}).get_text()
+            except AttributeError:
+                res = soup.find('div', attrs={'data-testid': 'hero-rating-bar__aggregate-rating__score'})
+                rating_imdb_raw = res.get_text() if res else None
+                res = soup.find('div', attrs={'data-testid': 'hero-rating-bar__aggregate-rating__score'})
+                rating_imdb_count_raw = res.next_sibling.next_sibling.get_text() if res else None
 
-        if rating_imdb_count_raw.endswith('K'):
-            rating_imdb_count = float(rating_imdb_count_raw[:-1]) * 1_000
-        elif rating_imdb_count_raw.endswith('M'):
-            rating_imdb_count = float(rating_imdb_count_raw[:-1]) * 1_000 * 1_000
+        rating_imdb = float(rating_imdb_raw) if rating_imdb_raw else None
+
+        if rating_imdb_count_raw:
+            if rating_imdb_count_raw.endswith('K'):
+                rating_imdb_count = float(rating_imdb_count_raw[:-1]) * 1_000
+            elif rating_imdb_count_raw.endswith('M'):
+                rating_imdb_count = float(rating_imdb_count_raw[:-1]) * 1_000 * 1_000
+            else:
+                rating_imdb_count = int(rating_imdb_count_raw.replace(',', ''))
         else:
-            rating_imdb_count = int(rating_imdb_count_raw.replace(',', ''))
-
+            rating_imdb_count = None
         return {'rating_imdb': rating_imdb,
                 'rating_imdb_count': rating_imdb_count}
 
@@ -147,9 +163,18 @@ class IMDBAssetScraper:
         try:
             soup_result = soup.select('div[class^="Storyline__StorylineWrapper"]')[0]
             soup_result = soup_result.div.div.div.get_text()
-            story_line = soup_result.replace("\n", "").replace('"', "").strip()
+            story_line: str = soup_result.replace("\n", "").replace('"', "").strip()
         except IndexError:
-            story_line = soup.find('div', {'id': 'titleStoryLine'}).div.p.span.get_text().strip()
+            try:
+                story_line = soup.find('div', {'id': 'titleStoryLine'}).div.p.span.get_text().strip()
+            except AttributeError:
+                try:
+                    soup_result = soup.select('div[data-testid^="storyline-plot-summary"]')[0]
+                    story_line = soup_result.div.div.get_text().strip()
+                except IndexError:
+                    soup_result = soup.select('ul[id="plot-summaries-content"]')[0]
+                    summaries = [p_tag.text for p_tag in soup_result.select('p')]
+                    story_line = '; '.join(summaries)
         return story_line
 
     def _parse_synopsis_from_soup(self, soup: BeautifulSoup) -> str:
@@ -163,25 +188,28 @@ class IMDBAssetScraper:
 
     @staticmethod
     def _parse_budget_from_soup(soup: BeautifulSoup) -> Optional[int]:
-        budget_raw = soup(text=re.compile('Budget'))
+        budget_raw = soup.find('li', {'data-testid': 'title-boxoffice-budget'})
         if budget_raw:
+            budget_raw_content = budget_raw.div.get_text().strip()
             try:
-                budget = budget_raw[0].next.replace('$', '').replace(',', '').strip()
+                budget: Optional[int] = \
+                    int(budget_raw_content.replace('$', '').replace(',', '').replace('(estimated)', '').strip())
             except TypeError:
                 budget = None
         else:
             budget = None
         return budget
 
+    def _parse_year_from_soup(self, soup: BeautifulSoup) -> int:
+        result = soup.find('meta', {'property': 'og:title'})
+        year = self._parse_year(result['content'])
+        return year
+
     @staticmethod
-    def _parse_year_from_soup(soup: BeautifulSoup) -> int:
-        title_orig = soup.find('meta', {'property': 'og:title'})['content']
-        return int(title_orig.split('(')[1].split(')')[0])
+    def _parse_year(raw_str: str) -> int:
+        content_last_brackets = raw_str.split('(')[-1].split(')')[0]
+        return int(content_last_brackets.replace("Video ", ""))
 
-    # < li
-    # role = "presentation"
-    #
-    # class ="ipc-inline-list__item" > < a href="/title/tt2120120/releaseinfo?ref_=tt_ov_rdat#releases" class ="ipc-link ipc-link--baseAlt ipc-link--inherit-color TitleBlockMetaData__StyledTextLink-sc-12ein40-1 rgaOW" > 2015 < / a > < span class ="TitleBlockMetaData__ListItemText-sc-12ein40-2 jedhex" > 2015 < / span > < / li >
     @staticmethod
     def _parse_fsk_from_soup(soup: BeautifulSoup) -> int:
         """fsk is the German required age to access an asset"""
@@ -201,9 +229,9 @@ class IMDBAssetScraper:
         return runtime
 
     @staticmethod
-    def _parse_awards_from_soup(soup: BeautifulSoup) -> dict:
+    def _parse_awards_from_soup(soup: BeautifulSoup) -> dict[str, Any]:
         search = soup.find_all('table', {'class': 'awards'})
-        awards: dict = {}
+        awards: dict[str, Any] = {}
         for award_table in search:
             cells = award_table.find_all('td')
             award_outcome_current = None
@@ -215,13 +243,14 @@ class IMDBAssetScraper:
                     award_category_current = cell.find('span').text
                 elif cell_htmlclass == 'award_description':
                     award_description = cell.text.split('\n')[1].strip()
-                    awards.setdefault(award_category_current, []).append((award_description, award_outcome_current))
+                    if award_category_current:  # from previous loop
+                        awards.setdefault(award_category_current, []). \
+                            append((award_description, award_outcome_current))
                 else:
                     raise Exception
         return awards
 
-    @staticmethod
-    def get_chart_ids(listing: str):
+    def get_chart_ids(self, listing: str) -> list[int]:
         listing_map = {'URL_TOP250': "https://www.imdb.com/chart/top?ref_=nv_mv_250",
                        'URL_BOTTOM100': "https://www.imdb.com/chart/bottom",
                        'URL_TOP250_ENGL': "https://www.imdb.com/chart/top-english-movies"}
@@ -229,6 +258,6 @@ class IMDBAssetScraper:
         if not listing_url:
             raise Exception(f"Not supported listing. Choose from {listing_map.keys()}!")
         else:
-            website = request.urlopen(listing_url).read()
+            website = request.urlopen(Request(listing_url, headers=self.header)).read()
             soup = BeautifulSoup(website, 'html.parser')
             return [int(finding['data-tconst'].strip('t')) for finding in soup.find_all('div', {'class': 'wlb_ribbon'})]
